use std::str::FromStr;
use std::sync::Arc;

use cidr::IpInet;
use serde::{Deserialize, Serialize};
use tokio::sync::Mutex;

type WgLink = String;
type WgPeer = String;

#[derive(Clone, Debug, Serialize, Deserialize)]
struct EventInfo {
    link: WgLink,
    cidr: IpInet,
}

#[derive(Clone, Debug, Serialize, Deserialize)]
enum Event {
    Add(EventInfo),
    Del(EventInfo),
}

struct EventParseError<T>(T);

impl FromStr for Event {
    type Err = EventParseError<&'static str>;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let re = regex::Regex::new(r"^(Deleted )?\d+: ([a-z0-9]+)\s+inet ((\d+\.?){4}(/\d+)?).*$")
            .unwrap();

        if let Some(captures) = re.captures(s) {
            let is_add = captures.get(1).is_none();
            let _is_delete = captures.get(1).is_some();
            let link: WgLink = captures.get(2).unwrap().as_str().into();
            let cidr: IpInet = IpInet::from_str(captures.get(3).unwrap().as_str()).unwrap();
            if is_add {
                Ok(Event::Add(EventInfo { link, cidr }))
            } else {
                Ok(Event::Del(EventInfo { link, cidr }))
            }
        } else {
            Err(Self::Err {
                0: "Line couldn't be parsed as event",
            })
        }

        //if re_send_add.is_match(s) {
        //    send_add = true;
        //} else if re_send_del.is_match(s) {
        //    send_del = true;
        //}
        //if send_add | send_del {
        //    cidr_ = re_extract_ip.captures(s).unwrap().get(1).unwrap().as_str();
        //    public_key = send_daemon_get_public_key(&iface).await;
        //} else {
        //    return Err<Self::Err>;
        //}
        //if send_add {
        //    let request = hyper::Request::builder()
        //        .method(hyper::Method::PATCH)
        //        .uri(format!("http://{peer}:port/wireguard/{iface}/peer/{public_key}/address", peer=cidr_, iface=iface, public_key=public_key))
        //        .body(hyper::Body::empty()).unwrap();
        //    hyper::Client::new().request(request).await.unwrap();
        //}
    }
}

type EventStore = Arc<Mutex<Vec<Event>>>;
